######################## case study ####################

library('scam')
library('psych')
library('ggplot2')
library('DescTools')

setwd('~/TCH-18-127R2_Codes_Revision')
source("Twostep.R")
source("StabilityConstr.R")
source("convScore.R")

###############################################################################
##############################  About the data  ###############################
###############################################################################

## >> load dataset 
df = data.frame(read.csv("WtgelData.csv",header = TRUE))
df<-df[,-6]
names(df)<-c('Hour','wt4','wt6','wt8','wt10')

#>>dimension#
m=dim(df)[2]-1; n=dim(df)[1]; k=1

#>>data structure: swelling ratio, vector of predictor#
ratio.df<-df/data.frame(rep(1,times=n),rep(df$wt4[1],times=n),rep(df$wt6[1],times=n),rep(df$wt8[1],times=n),rep(df$wt10[1],times=n))
names(ratio.df)<-c('Hour','wt4','wt6','wt8','wt10')
xvec=c(0.04,0.06,0.08,0.10)
tvec=ratio.df$Hour
data.df<-data.frame(rep(ratio.df$Hour,times=m),rep(xvec,each=n,times=1),
                    c(ratio.df$wt4,ratio.df$wt6,ratio.df$wt8,ratio.df$wt10))
names(data.df)<-c('time','X','Y')

##--------------------set estimat parameters -------------------##
fmr<-Y_r~theta1*(1-theta2*exp(-theta3*time))
y.df<-data.df
Xmat=array(rep(xvec,times=n*k),c(m,n,k))
Ymat=matrix(y.df$Y,ncol=n,byrow=TRUE)
theta_start<-list(theta1=1,theta2=0.5,theta3=1)
max.iter=100
tol=0.0005

########################################################################
##---------------- proposed unweighted estimation --------------------##
########################################################################

fit.mdl<-semi(fmr,data=y.df,Xmat=NULL,lambda=NULL,theta_start,max.iter=NULL,tol=NULL)

#######################################################################
##---------------- propsed weighted estimation ----------------------##
#######################################################################
Ymat.np<-matrix(fit.mdl$output.df$Y-fit.mdl$comp.p,ncol=n,byrow=TRUE)
new.mdl<-ConstrS(Ymat.np,Xmat,tvec,lambda=fit.mdl$sp,h=1,alpha=0.01,max.iter=100)

predbeta1.uw<-spline(tvec,fit.mdl$beta,method='natural',n=1000)
predbeta1.w<-spline(tvec,new.mdl$beta,method='natural',n=1000)

fun.predbeta0<-function(time){fit.mdl$theta[1]*(1-fit.mdl$theta[2]*exp(-fit.mdl$theta[3]*time))}
predbeta0<-curve(fun.predbeta0(x),tvec[1],tvec[n])

ycurve.w<-as.vector(matrix(predbeta1.w$y,nrow=length(predbeta1.w$y))%*%matrix(xvec,ncol=m)+matrix(rep(fun.predbeta0(predbeta1.w$x),times=m),ncol=m,byrow=FALSE))

newdata<-data.frame('time'=rep(predbeta1.w$x, times=m),
                          'X'=rep(xvec,each=length(predbeta1.w$x)))

     ##==== compare the proposed weighted vs proposed unweighted====##
     ##convscore##
      convScore0<-convScore(tvec,fit.mdl$beta,new.mdl$conv.time,tvec[n])
      rbind(convScore0,new.mdl$convScore)
      
     ##residuals##
      rbind(sum(fit.mdl$residuals^2),sum(new.mdl$residuals^2))

########################################################################      
##-------------------------- Conventional VCM ------------------------##
########################################################################
scam0.mdl <- scam(Y~s(time,k=n,bs="cr",m=2,id=1)+s(time,by=X,m=2,id=2,k=n,bs='cr'),
                 family=gaussian(link="identity"),data=data.df,not.exp=FALSE)

   ##=======compare the SSE==========##
    c("sse",rbind(sum(abs(scam0.mdl$residuals)^2),sum(abs(new.mdl$residuals)^2)))
    c("MAE",rbind(mean(abs(scam0.mdl$residuals)),mean(abs(new.mdl$residuals))))
    c("Max abs",rbind(max(abs(scam0.mdl$residuals)),max(abs(new.mdl$residuals))))

########################################################################
##----------------------- SCAM:monotonic beta0 -----------------------##
########################################################################
scam1.mdl <- scam(Y~s(time,k=n,bs="mpi",m=2)+s(time,by=X,m=2,k=n,bs='cr'),
          family=gaussian(link="identity"),data=data.df,not.exp=FALSE)

   ##===== compare the SSE======##
    c("sse",rbind(sum(abs(scam1.mdl$residuals)^2),sum(abs(new.mdl$residuals)^2)))
    c("MAE",rbind(mean(abs(scam1.mdl$residuals)),mean(abs(new.mdl$residuals))))
    c("Max abs",rbind(max(abs(scam1.mdl$residuals)),max(abs(new.mdl$residuals))))

