######################### Case Study Plots #########################

library('ggplot2')
library('scam')

setwd('~/TCH-18-127R2_Codes_Revision')
load('CaseStudy_data.Rdata')


###################################################################
##============ About the proposed (constrained) method===========##
###################################################################

##--- plot the convergence of parametric part estimation----##
plot(c(1:fit.mdl$iter.num),fit.mdl$theta.conv,ylim=range(fit.mdl$theta.conv)+c(0,0.5),type='o',col='black',lty=1,lwd=1.4)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)

##--- plot the estimated parametric part----##
curve(fun.predbeta0(x),tvec[1],tvec[n],ylim=range(predbeta0$y)+c(0,0.02),lwd=1.6)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)


##--- plot the estimated nonparametric part ----##  
ggplot(data=proposed.df)+theme_bw()+
  geom_line(aes(y=beta,x=time,color=factor(label)),size=1)+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='',
                     labels=c("proposed (unconstrained)","proposed (constrained)"),
                     values=c("0"="black","1"="red"))+
  theme_bw() + 
  theme(legend.position = c(0.6, 0.35),
        panel.border = element_rect(colour = "black",size=1.08),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(), 
        axis.line = element_line(colour = "black"),
        strip.background = element_blank(),
        text = element_text(size=12),
        axis.text= element_text(size=10, hjust=1,face='bold'))

##--- plot the lower and upper bound----##
bound.df<-data.frame("Y"=c(new.mdl$bound$lower,new.mdl$bound$upper,fit.mdl$output.df$Y-fit.mdl$comp.p,fit.mdl$comp.np),
                     "X"=rep(fit.mdl$output.df$X,4),
                     "time"=rep(fit.mdl$output.df$time,4),
                     "type"=rep(c(1,2,3,4),each=length(fit.mdl$fitted))
)

id=4; # id=1,2,3,4 for each group
select.df<-bound.df[bound.df$X==xvec[id],]
select.col<-c("black","red","blue","green4")[id]
select.shp<-list(c("1"=32,"2"=32,"3"=15,"4"=0),
                 c("1"=32,"2"=32,"3"=15,"4"=0),
                 c("1"=32,"2"=32,"3"=17,"4"=2),
                 c("1"=32,"2"=32,"3"=25,"4"=6))

ggplot(data=select.df)+
  geom_point(aes(y=Y,x=time,shape=factor(type)),color=select.col,fill=select.col)+
  geom_line(aes(y=Y,x=time,linetype=factor(type),color=factor(type)))+
  labs(title=" ", y=" ", x="Time (hour)", caption=" ")+
  scale_color_manual(name='',
                     labels=c("Lower bound","Upper bound","Observed","Fitted"),
                     values=c("1"="gray","2"="gray","3"=select.col,"4"=select.col)
  )+
  scale_shape_manual(name='',
                     labels=c("Lower bound","Upper bound","Observed","Fitted"),
                     values=select.shp[[id]]
  )+
  scale_linetype_manual(name='',
                        labels=c("Lower bound","Upper bound","Observed","Fitted"),
                        values=c("1"="solid","2"="solid","3"="solid","4"="longdash")
  )+
  theme_bw() + theme(legend.position = c(0.85, 0.35),
                     panel.border = element_rect(colour = "black",size=1.08),
                     panel.grid.major = element_blank(),
                     panel.grid.minor = element_blank(), 
                     axis.line = element_line(colour = "black"),
                     strip.background = element_blank(),
                     text = element_text(size=12),
                     axis.text= element_text(size=10, hjust=1,face='bold'))

#################################################################
##===== compare the proposed:constrained vs unconstrained =====##
#################################################################

##----------------plot the residuals---------------------
uw.resid<-data.frame('resid'=c(fit.mdl$residuals,new.mdl$residuals),
                     'time'=rep(data.df$time,times=2),
                     'X'=rep(data.df$X,times=2),
                     'method'=rep(c(0,1),each=n*m))
ggplot(data=uw.resid)+
  geom_point(aes(y=resid,x=time,color=factor(method)))+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='methods',
                     labels=c("proposed (unconstrained)","proposed (constrained)"),
                     values=c("0"="blue","1"="red"))+
  theme(#legend.position = c(0.85, 0.2),
    
    text = element_text(size=12),
    axis.text= element_text(size=12, hjust=1,face='bold'))+
  facet_grid(X~.)

##---------------plot beta1 -----------------------
proposed.df<-data.frame('beta'=c(predbeta1.uw$y,predbeta1.w$y),
                        'time'=rep(predbeta1.uw$x,times=2),
                        'label'=rep(c(0,1),each=1000))

ggplot(data=proposed.df)+theme_bw()+
  geom_line(aes(y=beta,x=time,color=factor(label),linetype=factor(label)),size=1)+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='',
                     labels=c("Proposed (unconstrained)","Proposed (constrained)"),
                     values=c("0"="black","1"="red"))+
  scale_linetype_manual(name='',
                        labels=c("Proposed (unconstrained)","Proposed (constrained)"),
                        values=c("0"="twodash","1"="solid"))+
  theme_bw()+
  theme(legend.position = c(0.65, 0.5),
        panel.border = element_rect(colour = "black",size=1.08),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(), 
        axis.line = element_line(colour = "black"),
        strip.background = element_blank(),
        text = element_text(size=12),
        axis.text= element_text(size=10, hjust=1,face='bold'))



##---------------plot fitted value -----------------------
uw.results<-data.frame('Y'=c(data.df$Y,fit.mdl$fitted,new.mdl$fitted+fit.mdl$comp.p),
                       'time'=rep(data.df$time,times=3),
                       'X'=rep(data.df$X,times=3),
                       'method'=rep(c(0,1,2),each=n*m))

ggplot(data=uw.results)+
  geom_point(aes(y=Y,x=time,color=factor(method)))+
  geom_line(aes(y=Y,x=time,color=factor(method)))+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='methods',
                     labels=c("observed","proposed (unconstrained)","proposed (constrained)"),
                     values=c("0"="black","1"="blue","2"="red"))+
  theme(#legend.position = c(0.85, 0.2),
    text = element_text(size=12),
    axis.text= element_text(size=12, hjust=1,face='bold'))+
  facet_grid(X~.)

################################################################
##==== compare the proposed contrained vs conventional VCM====##
################################################################

##------------compare the beta0--------------
plot(scam0.mdl,se=FALSE,select=1,ylim=c(-0.2,0.2),col="blue",ylab='beta0',lty=4,lwd=3)
points(predbeta0$x,predbeta0$y-scam0.mdl$coefficients[1],type='l',col='red',lty=1,lwd=2)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)
legend(120,-0.1, legend=c("conventional VCM","Proposed (constrained)"),
       col=c("blue","red"), lty=c(4,1), lwd=c(2,2),cex=0.8)


##------------compare the beta1-----------------
plot(scam0.mdl,se=FALSE,select=2,ylim=c(-1.1,0.15),col="blue",ylab='beta1',lty=4,lwd=3)
points(predbeta1.w$x,predbeta1.w$y,type='l',col='red',lty=1,lwd=2)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)
legend(120,-0.75, legend=c("conventional VCM","Proposed (constrained)"),
       col=c("blue","red"), lty=c(4,1), lwd=c(2,2),cex=0.8)


##-------------- plot the residuals------------------
scam0.resid<-data.frame('resid'=c(scam0.mdl$residuals,new.mdl$residuals),
                        'time'=rep(data.df$time,times=2),
                        'X'=rep(data.df$X,times=2),
                        'id'=rep(c(1:n),times=m*2),
                        'method'=rep(c(0,1),each=n*m))
ggplot(data=scam0.resid)+
  geom_point(aes(y=resid,x=id,color=factor(method),shape=factor(method)))+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='methods',
                     labels=c("conventional VCM","proposed"),
                     values=c("0"="blue","1"="red"))+
  scale_shape_manual(name='methods',
                     labels=c("conventional VCM","proposed"),
                     values=c("0"=17,"1"=16))+
  theme(#legend.position = c(0.85, 0.2),
    text = element_text(size=12),
    axis.text= element_text(size=12, hjust=1,face='bold'))+
  facet_grid(X~.)

##----------------plot the fitted---------------------

ycurve.scam0<-predict(scam0.mdl,newdata=newdata)

scam0.results<-data.frame('Y'=c(data.df$Y,ycurve.scam0,ycurve.w),
                          'time'=c(data.df$time,rep(predbeta1.w$x,times=m*2)),
                          'X'=c(data.df$X,rep(xvec,each=length(predbeta1.w$x),times=2)),
                          #'ltype'=c(rep(0,times=m*n),rep(1,times=2*m*length(predbeta1.w$x))),
                          'ctype'=c(rep(0,times=m*n),rep(c(1,2),each=m*length(predbeta1.w$x)))
)

ggplot(data=scam0.results)+
  geom_point(aes(y=Y,x=time,color=factor(ctype),shape=factor(ctype)))+
  geom_line(aes(y=Y,x=time,color=factor(ctype),linetype=factor(ctype)))+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='',
                     labels=c("observed","conventional VCM","proposed"),
                     values=c("0"="black","1"="blue","2"="red"))+
  scale_shape_manual(name='',
                     labels=c("observed","conventional VCM","proposed"),
                     values=c("0"=19,"1"=32,"2"=32))+
  scale_linetype_manual(name='',
                        labels=c("observed","conventional VCM","proposed"),
                        values=c("0"="blank","1"="solid","2"="solid"))+
  theme(#legend.position = c(0.85, 0.2),
    text = element_text(size=12),
    axis.text= element_text(size=12, hjust=1,face='bold'))+
  facet_grid(X~.)


###################################################################
##===compare the proposed contrained vs SCAM:monotonic beta0 ====##
###################################################################

##--------------compare the beta0-----------------
plot(scam1.mdl,se=FALSE,select=1,ylim=c(0.7,1.2),col="blue",ylab='beta0',lwd=2,lty=4)
points(predbeta0$x,predbeta0$y-scam1.mdl$coefficients[1],type='l',col='red',lwd=2,lty=1)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)
legend(120,0.85, legend=c("SCAM (constrained)","Proposed (constrained)"),
       col=c("blue","red"), lty=c(4,1), lwd=c(2,2),cex=0.8)


##------------- compare the beta1----------------
plot(scam1.mdl,se=FALSE,select=2,ylim=c(-0.8,0.15),col="blue",ylab='beta1',lwd=4,lty=4)
points(predbeta1.w$x,predbeta1.w$y,type='l',col='red',lwd=2,lty=1)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)
legend(120,-0.45, legend=c("SCAM (constrained)","Proposed (constrained)"),
       col=c("blue","red"), lty=c(4,1), lwd=c(2,2),cex=0.8)


##---------- plot the residuals ----------------
scam1.resid<-data.frame('resid'=c(scam1.mdl$residuals,new.mdl$residuals),
                        'time'=rep(data.df$time,times=2),
                        'X'=rep(data.df$X,times=2),
                        'id'=rep(c(1:n),times=m*2),
                        'method'=rep(c(0,1),each=n*m))
ggplot(data=scam1.resid)+
  geom_point(aes(y=resid,x=id,color=factor(method),shape=factor(method)))+
  #geom_line(aes(y=resid,x=time,color=factor(method)))+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='method',
                     labels=c("constrained SCAM","proposed"),
                     values=c("0"="blue","1"="red"))+
  scale_shape_manual(name='method',
                     labels=c("constrained SCAM","proposed"),
                     values=c("0"=17,"1"=16))+
  theme(#legend.position = c(0.85, 0.2),
    text = element_text(size=12),
    axis.text= element_text(size=12, hjust=1,face='bold'))+
  facet_grid(X~.)

##--------------- plot the fitted values ------------------------

ycurve.scam1<-predict(scam1.mdl,newdata=newdata)

scam1.results<-data.frame('Y'=c(data.df$Y,ycurve.scam1,ycurve.w),
                          'time'=c(data.df$time,rep(predbeta1.w$x,times=m*2)),
                          'X'=c(data.df$X,rep(xvec,each=length(predbeta1.w$x),times=2)),
                          #'ltype'=c(rep(0,times=m*n),rep(1,times=2*m*length(predbeta1.w$x))),
                          'ctype'=c(rep(0,times=m*n),rep(c(1,2),each=m*length(predbeta1.w$x)))
)


ggplot(data=scam1.results)+
  geom_point(aes(y=Y,x=time,color=factor(ctype),shape=factor(ctype)))+
  geom_line(aes(y=Y,x=time,color=factor(ctype),linetype=factor(ctype)))+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='',
                     labels=c("observed","constrained SCAM","proposed"),
                     values=c("0"="black","1"="blue","2"="red"))+
  scale_shape_manual(name='',
                     labels=c("observed","constrained SCAM","proposed"),
                     values=c("0"=19,"1"=32,"2"=32))+
  scale_linetype_manual(name='',
                        labels=c("observed","constrained SCAM","proposed"),
                        values=c("0"="blank","1"="solid","2"="solid"))+
  theme(#legend.position = c(0.85, 0.2),
    text = element_text(size=12),
    axis.text= element_text(size=12, hjust=1,face='bold'))+
  facet_grid(X~.)


