#------------------------simulation----------------------------------#
#install.packages('scam')
#install.packages('psych')
#install.packages('DescTools',dependencies = TRUE)
#install.packages("ggplot2", repos = c("http://rstudio.org/_packages","http://cran.rstudio.com"),dependencies = TRUE)

library('scam')
library('psych')
library('ggplot2')
library('DescTools')

setwd('~/TCH-18-127R2_Codes_Revision')
source("twostep.R")
source("StabilityConstr.R")
source("convScore.R")

##===========change values of parameters here ================
m=5;n=30;k=1;
xvec<-c(0.05,0.19,0.36,0.72,0.91)
tvec<-seq(from=0,to=15,length.out=n)
theta1=3; theta2=1.2; theta3=1.5;
a=5;b=0.4;
sigma2<-0.3
#=============================================================

##------- true functions---------
Xmat<-array(rep(xvec,times=n),c(m,n,k))
beta0.func<-function(time){time^theta1*exp(-time/theta2)+theta3}
beta1.func<-function(time){a*time/sqrt(b+time^2)}
Ymat<-matrix(xvec,nrow=m)%*%beta1.func(tvec)+matrix(rep(beta0.func(tvec),times=m),nrow=m,byrow=TRUE)

##---- simulated data----
set.seed(100)
rn<-rnorm(n*m,0,sigma2);
Ymat<-Ymat+matrix(rn,nrow=m)

truey.list<-list()
for (i in c(1:m)){
  truey.list[[i]]<-xvec[i]*beta1.true$y+beta0.true$y;
}
curve.df<-data.frame("Y"=c(as.vector(do.call(cbind,truey.list)),as.vector(t(Ymat))),
                     "time"=c(rep(beta0.true$x,times=m),rep(tvec,times=m)),
                     "X"=c(rep(xvec,each=length(beta0.true$x)),rep(xvec,each=n)),
                     "noise"=c(rep(0,m*length(beta0.true$x)),rep(1,m*n))
)

simul.df<-data.frame('time'=rep(tvec,times=m),
                  'X'=rep(xvec,each=n),
                  'Y'=as.vector(t(Ymat)))

#===========================fit with proposed weighted=====================
fmr<-Y_r~time^theta1*exp(-time/theta2)+theta3
theta_init<-list(theta1=1,theta2=1,theta3=1)
max.iter=100
tol=0.005
alpha=0.01
h=1

##---------------- proposed unweighted estimation --------------------##
S1.mdl<-semi(fmr,data=simul.df,Xmat=Xmat,lambda=NULL,theta_start=theta_init,max.iter=max.iter,tol=tol)

S1.beta1<-spline(tvec,S1.mdl$beta,method='natural',n=1000) 

##---------------- proposed weighted estimation --------------------##
S1ymat.np<-matrix(S1.mdl$output.df$Y-S1.mdl$comp.p,ncol=n,byrow=TRUE)
S2.mdl<-ConstrS(S1ymat.np,Xmat,tvec,lambda=S1.mdl$sp,h=h,alpha=alpha,max.iter=max.iter)

S2.beta1<-spline(tvec,S2.mdl$beta,method='natural',n=1000) 

S1.beta0func<-function(time){time^S1.mdl$theta[1]*exp(-time/S1.mdl$theta[2])+S1.mdl$theta[3]}
S2.beta0<-curve(S1.beta0func(x),tvec[1],tvec[n])


newdata<-data.frame('time'=rep(S2.beta1$x, times=m),
                    'X'=rep(xvec,each=length(S2.beta1$x)))

ycurve.prop<-as.vector(matrix(S2.beta1$y,nrow=length(S2.beta1$y))%*%matrix(xvec,ncol=m)+matrix(rep(S1.beta0func(S2.beta1$x),times=m),ncol=m,byrow=FALSE))

##--------------conventional VCM(SCAM)------------------------

S03.mdl <- scam(Y~s(time,m=2,k=n,bs="cr")+s(time,by=X,m=2,k=n,bs="cr"),                  family=gaussian(link="identity"),data=simul.df,not.exp=FALSE)

    