#####################    Simulation plot    #################################
library(ggplot2)
library('scam')

setwd('~/TCH-18-127R2_Codes_Revision')
load('Simulation_data.Rdata')


###############################################################################
########################  About the generated data  ###########################
###############################################################################

##-- plot the true coefficient --
beta0.true<-curve(beta0.func(x),tvec[1],tvec[n])
beta1.true<-curve(beta1.func(x),tvec[1],tvec[n])

##--plot the generated data---

ggplot(data=curve.df)+
  geom_point(aes(y=Y,x=time,color=factor(X),shape=factor(noise)))+
  geom_line(aes(y=Y,x=time,linetype=factor(noise),color=factor(X)),size=0.7)+
  #geom_vline(xintercept = S2.mdl$conv.time, linetype="dotted",color = "black", size=0.7)+
  labs(title=" ", y=" ", x="", caption=" ")+
  scale_color_discrete(name='X values',labels=xvec)+
  scale_shape_manual(name='Data type',
                     values=c("0"=32,"1"=19),
                     labels=c("true","with noise")
  )+
  scale_linetype_manual(name='Data type',
                        values=c("0"="solid","1"="blank"),
                        labels=c("true","with noise")
  )+
  theme_bw() + theme(#legend.position = c(0.85, 0.2),
    panel.border = element_rect(colour = "black",size=1),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(), 
    axis.line = element_line(colour = "black"),
    strip.background = element_blank(),
    text = element_text(size=10),
    axis.text= element_text(size=10, hjust=1,face='bold'))+
  facet_grid(X~.)


###############################################################################
###########################  Compare the models: ##############################
#######   proposed (unconstrained/constrained), conventional VCM   ############
###############################################################################

##---------------------compare the beta0----------------------------- 
##---(proposed(unconstrained),conventional VCM, Actual)--------------

plot(S03.mdl,se=FALSE,select=1,ylim=range(beta0.true$y-S03.mdl$coefficients[1])+c(-1,1),col="blue",ylab='beta0',lty=3,lwd=2)
points(beta0.true$x,beta0.true$y-S03.mdl$coefficients[1],type='l',col='black',lty=1,lwd=2)
points(S2.beta0$x,S2.beta0$y-S03.mdl$coefficients[1],type='l',col='orange',lty=5,lwd=2)
#points(S2.beta0$x,S2.beta0$y-S03.mdl$coefficients[1],type='l',col='red',lty=1,lwd=2)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)
legend(7.2,2.5, legend=c("Conventional VCM","True","Proposed (unconstrained)"),
       col=c("blue","black",'orange'), lty=c(3,1,5), lwd=c(2,2,2),cex=0.8)

##-------------------------compare the beta1------------------------- 
##----(proposed(unconstrained),conventional VCM,Actual)--------------

plot(S03.mdl,se=FALSE,select=2,ylim=range(beta1.true$y)+c(-1,1),col="blue",ylab='beta1',lty=3,lwd=2)
points(beta1.true$x,beta1.true$y,type='l',col='black',lty=1,lwd=2)
points(S2.beta1$x,S1.beta1$y,type='l',col='orange',lty='dashed',lwd=2)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)
legend(7.5,1, legend=c("Conventional VCM","True","Proposed (unconstrained)"),
       col=c("blue","black",'orange'), lty=c(3,1,5), lwd=c(2,2,2),cex=0.8)

##------------------compare the beta0------------------------------- 
##--------(proposed(constrained),conventional VCM,True)--------------

plot(S03.mdl,se=FALSE,select=1,ylim=range(beta0.true$y-S03.mdl$coefficients[1])+c(-1,1),col="blue",ylab='beta0',lty=3,lwd=2)
points(beta0.true$x,beta0.true$y-S03.mdl$coefficients[1],type='l',col='black',lty=1,lwd=2)
points(S2.beta0$x,S2.beta0$y-S03.mdl$coefficients[1],type='l',col='red',lty=5,lwd=2)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)
legend(7.5,2.5, legend=c("Conventional VCM","True","Proposed (constrained)"),
       col=c("blue","black","red"), lty=c(3,1,5), lwd=c(2,2,2),cex=0.8)

##-----------------------compare the beta1------------------------------ 
##-------------(proposed(constrained),conventional VCM,True)------------

plot(S03.mdl,se=FALSE,select=2,ylim=range(beta1.true$y)+c(-1,1),col="blue",ylab='beta1',lty=3,lwd=2.5)
points(beta1.true$x,beta1.true$y,type='l',col='black',lty=1,lwd=2)
points(S2.beta1$x,S2.beta1$y,type='l',col='red',lty=5,lwd=2)
abline(v=S2.mdl$conv.time,col='gray45',lty=1,lwd=1.2)
box(lwd = 2) 
axis(1, font=2)
axis(2, font=2)
legend(7.5,1, legend=c("Conventional VCM","True", "Proposed (constrained)"),
       col=c("blue","black","red"),lty=c(3,1,5),lwd=c(2,2,2.5), cex=0.8)

##-------------------compare the residuals------------------------
err_list=c(sum(abs(S03.mdl$residuals)^2),sum(abs(S2.mdl$residuals)^2),mean(abs(S03.mdl$residuals)),
          mean(abs(S2.mdl$residuals)),max(abs(S03.mdl$residuals)),max(abs(S2.mdl$residuals)))
err_table<-matrix(err_list, ncol=2, byrow=TRUE)
colnames(err_table)<-c("Conventional VCM", "Proposed(Constrained)")
rownames(err_table)<-c("Sum of squared error","Mean absolute error","Max absolute error")
err_table


##---------------plot the residuals --------------------
S.resid<-data.frame('resid'=c(S03.mdl$residuals,S2.mdl$residuals),
                    'time'=rep(simul.df$time,times=2),
                    'X'=rep(simul.df$X,times=2),
                    'method'=rep(c(0,1),each=n*m))
ggplot(data=S.resid)+
  geom_point(aes(y=resid,x=time,color=factor(method),shape=factor(method)))+
  #geom_line(aes(y=resid,x=time,color=factor(method)))+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='methods',
                     labels=c("unconstrained SCAM","proposed"),
                     values=c("0"="blue","1"="red"))+
  scale_shape_manual(name='methods',
                     labels=c("unconstrained SCAM","proposed"),
                     values=c("0"=17,"1"=19))+
  theme(#legend.position = c(0.85, 0.2),
    text = element_text(size=12),
    axis.text= element_text(size=12, hjust=1,face='bold'))+
  facet_grid(X~.)


##----------- plot the fitted values-----------------
ycurve.scam<-predict(S03.mdl,newdata=newdata)

S.results<-data.frame('Y'=c(simul.df$Y,curve.df$Y[curve.df$noise==0],ycurve.scam,ycurve.prop),
                      'time'=c(simul.df$time,curve.df$time[curve.df$noise==0],rep(S2.beta1$x,times=m*2)),
                      'X'=c(simul.df$X,curve.df$X[curve.df$noise==0],rep(xvec,each=length(S2.beta1$x),times=2)),
                      #'ltype'=c(rep(0,times=m*n),rep(1,times=2*m*length(S2.beta1$x))),
                      'ctype'=c(rep(0,times=m*n),rep(1,times=length(curve.df$time[curve.df$noise==0])),rep(c(2,3),each=m*length(S2.beta1$x)))
)

ggplot(data=S.results)+
  geom_point(aes(y=Y,x=time,color=factor(ctype),shape=factor(ctype)))+
  geom_line(aes(y=Y,x=time,color=factor(ctype),linetype=factor(ctype)))+
  labs(title=" ", y=" ", x="Time(hour)", caption=" ")+
  scale_color_manual(name='',
                     labels=c("observed","true","conventional VCM","proposed"),
                     values=c("0"="black","1"="black","2"="blue","3"="red"))+
  scale_shape_manual(name='',
                     labels=c("observed","true","conventional VCM","proposed"),
                     values=c("0"=19,"1"=32,"2"=32,"3"=32))+
  scale_linetype_manual(name='',
                        labels=c("observed","true","conventional VCM","proposed"),
                        values=c("0"="blank","1"="solid","2"="dashed","3"="dashed"))+
  theme(#legend.position = c(0.85, 0.2),
    text = element_text(size=12),
    axis.text= element_text(size=12, hjust=1,face='bold'))+
  facet_grid(X~.)

