#' Compute the convengence score of a curve within a specified range
#' 
#' @param x The x-axis values of the curve
#' @param y The y-axis values of the curve
#' @param t0 The start point of convergence range
#' @param tn The end point of convergence range
#' 
#' @return A numeric value measuring the convergence of the curve within the specified range
#' 
#' @export
convScore<-function(x,y,t0,tn){
  library('DescTools')
  fun0<-splinefun(x,y,method='natural')
  ls(envir = environment(fun0))
  splinecoef <- get("z", envir = environment(fun0))
  c0<-curve(abs(fun0(x,deriv=1)),t0,tn);
  convScore0<-AUC(c0$x,c0$y, method='spline')
  return(convScore0)
}
